# -*- coding: utf-8 -*-
# whoactive.py - List users currently active on the system.
#
# Copyright (C) 2008, 2009 Fabian Knittel <fabian.knittel@avona.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import utmp
import UTMPCONST
import pwd
import proctable
import os
import errno

EXE_WHITELIST = ['/usr/bin/screen']
CMDLINE_WHITELIST = [
        ['/bin/bash', '/usr/NX/bin/nxnode', '--startsession']
    ]


def get_utmp_sessions():
    """Uses utmp to compile a list of users with active sessions."""
    active = set()
    record = utmp.UtmpRecord()
    for ut in record:
        if ut.ut_type == UTMPCONST.USER_PROCESS and ut.ut_pid > 0:
            try:
                os.kill(ut.ut_pid, 0)
            except OSError, err:
                if err.errno == errno.ESRCH:
                    # PID does not exist? Stale entry.
                    continue
            active.add(ut.ut_user)
    record.endutent()
    return active

def read_valid_shells():
    return filter(lambda line: not line.startswith('#'),
            open('/etc/shells', 'r').read().splitlines())

def has_valid_tty(proc):
    tty_devs = set(proc.tty_devices)
    # Ignore ptmx, as it's only a managing interface, not a real TTY.
    tty_devs.discard('/dev/ptmx')
    # Attached to any TTYs?
    return len(tty_devs) > 0

def get_process_sessions():
    """Uses proctable to compile a list of users with active sessions."""
    active = set()
    shells = read_valid_shells()
    procs = proctable.get_processes()
    for proc in procs.values():
        if (has_valid_tty(proc) and (proc.exe in shells)) \
                or proc.exe in EXE_WHITELIST \
                or proc.cmdline in CMDLINE_WHITELIST:
            login_name = pwd.getpwuid(proc.ruid)[0]
            # Add to list of active procs.
            active.add(login_name)
    return active

def get_active_sessions():
    """Uses various approaches to gather a list of users with active
    sessions."""
    return get_utmp_sessions().union(get_process_sessions())

# vim:set textwidth=80 tabstop=8 softtabstop=4 shiftwidth=4 et:
